import time
from datetime import datetime


class Coffee:

    def __init__(self, name, price, ingredients):
        self.name: str = name
        self.price: int = price
        self.ingredients: str = ingredients

    def __str__(self):
        return f"{self.name}({self.price}元): {self.ingredients}"


class Latte(Coffee):

    def __init__(self):
        super().__init__(name='拿铁', price=20, ingredients='浓缩咖啡，牛奶，糖')


class Americano(Coffee):
    def __init__(self):
        super().__init__(name='美式', price=15, ingredients='浓缩咖啡')


class CoffeeMachine:

    def __init__(self):
        self.records = []
        self.products = [Latte(), Americano()]

    def display(self):
        """展示咖啡"""
        print('------ 展示咖啡 ------')
        for product in self.products:
            print(product)

    def select_coffee(self) -> Coffee:
        """选择咖啡"""
        print('------ 选择咖啡 ------')
        while True:
            choice = input('请输入数字选择咖啡 0：拿铁，1：美式 ')
            if choice in ('0', '1'):
                return self.products[int(choice)]
            else:
                continue


    def input_money(self, coffee) -> int:
        """投入钱币"""
        print('------ 投入钱币 ------')
        while True:
            try:
                coin = input(f'请投入{coffee.price}元 ')

                if int(coin) < coffee.price:
                    print('投币金额不够，已退回。请重新投币')
                    continue

                else:
                    return int(coin)
            except Exception as e:
                print('输入金额不合法，请重新输入')


    def make_coffee(self, coffee, money):
        """制作咖啡"""
        print('------ 制作咖啡 ------')
        # 制作
        print('制作中，请稍候.....')
        time.sleep(2)
        print(f'{coffee.name} ☕️咖啡制作完成！')

        # 记录 2023-03-24 16:24:49
        tm = datetime.now().strftime('%Y-%m-%d %H:%M:%S')
        record = f"{tm} 订单：{coffee.name}咖啡️，支付{money}元，找零{money - coffee.price}元"
        self.records.append(record)

    def print_records(self):
        """打印记录"""
        print('------ 打印记录 ------')
        for rec in self.records:
            print(rec)


if __name__ == '__main__':
    machine = CoffeeMachine()

    while True:
        # 展示咖啡
        machine.display()
        # 选择咖啡
        selected_coffee = machine.select_coffee()
        # 投入钱币
        coin = machine.input_money(coffee=selected_coffee)
        # 制作咖啡
        machine.make_coffee(coffee=selected_coffee, money=coin)

        answer = input('是否继续购买？Y：是，N：否')
        if answer == 'Y':
            continue
        else:
            break

    # 打印记录
    machine.print_records()
